import 'package:flutter/material.dart';
import 'package:sw_component/components/widget/better_button.dart';
import 'package:sw_component/components/widget/date_picker.dart';
import 'package:sw_component/components/widget/error_box.dart';
import 'package:sw_component/components/widget/image_compat.dart';
import 'package:sw_component/components/widget/item_form_field.dart';
import 'package:sw_component/components/widget/material_theme_wrapper.dart';
import 'package:sw_component/components/widget/result_screen.dart';
import 'package:sw_component/components/widget/slim_appbar.dart';
import 'package:sw_core/entity_auth/auth_step_params.dart';
import 'package:sw_core/entity_auth/login_security_details.dart';
import 'package:sw_core/entity_auth/password_step.dart';
import 'package:sw_core/entity_auth/security_details.dart';
import 'package:sw_core/entity_auth/user_pass.dart';
import 'package:sw_core/entity_servicer/auth_params.dart';
import 'package:sw_core/entity_servicer/servicer.dart';
import 'package:sw_core/entity_theme/params_loan_connect.dart';
import 'package:sw_core/interface/controller/i_controller_auth.dart';
import 'package:sw_core/interface/controller/i_controller_polled_auth.dart';
import 'package:sw_core/interface/service/i_service_token.dart';
import 'package:sw_core/request/request_auth.dart';
import 'package:sw_core/request/request_auth_add_steps.dart';
import 'package:sw_core/tool/inject.dart';
import 'package:sw_core/navigation/nav.dart';
import 'package:sw_core/tool/observer.dart';
import 'package:sw_core/tool/state.dart';
import 'package:sw_loan_connect/input/event_handler_loan_connect.dart';
import 'package:sw_loan_connect/tool/servicer_id.dart';

class LayoutLoginB extends StatelessWidget {
  LayoutLoginB({Key? key}) : super(key: key);

  final ParamsLoanConnect params = get<ParamsLoanConnect>();
  final BlueDatePicker _datePicker = BlueDatePicker(showLabel: true, label: "Date of Birth");
  final _userBox = BlueItemFormField(
    label: "Username",
  ).obs;
  final BlueItemFormField _passwordBox = BlueItemFormField(
    label: "Password",
    obscureText: true,
  );
  final scrollController = ScrollController();
  final controllerAuth = get<IControllerAuth>();
  final controllerPolledAuth = getSafe<IControllerPolledAuth>();
  final serviceToken = get<IServiceToken>();

  final Servicer servicer = get<Servicer>();

  final List<TextEditingController> _authParams = [];
  final Map<String, String> _assets = {
    "9422554f-0170-4afe-b8df-12aaecc3fdef": "logo_aes.png",
    "e32be248-d43f-4dd4-9e34-05c7b8ab1888": "logo_firstmark.png",
    "bb3e39e7-9dd6-461e-a4f4-c95a1a1e80ef": "logo_nelnet.png",
    "e35a6454-ca66-4b35-a7e7-afb25d03c424": "logo_suntrust.png",
  };

  final _errorBoxVisibility = false.obs;
  final _sendButtonClicked = false.obs;
  final _isUserIDInsteadOfUserName = false.obs;
  final _canLaunchForgotPassword = true.obs;

  @override
  Widget build(BuildContext context) {
    return MaterialThemeWrapper(
      child: _buildBody(context),
    );
  }

  Widget _buildBody(BuildContext context) => Flex(
        direction: Axis.vertical,
        children: [
          Expanded(
            child: Observer(() => _checkAuthStep(context)),
          )
        ],
      );

  _onBackPressed(BuildContext context) {
    controllerAuth.authStep.value = AuthStep.STEP_UNAUTHORIZED;
    controllerAuth.security.value = SecurityStep();
    Navigator.of(context, rootNavigator: true).pop(context);
  }

  Widget _checkAuthStep(BuildContext context) {
    _applyRules();
    switch (controllerAuth.authStep.value) {
      case AuthStep.STEP_UNAUTHORIZED:
        return _initialState(false);
      case AuthStep.STEP_SECURITY:
        return _authStepSecurity(context, false);
      case AuthStep.STEP_AUTHORIZED:
        return _authCompleted(context);
      case AuthStep.AUTH_ERROR:
        return _authError(context);
    }
  }

  _applyRules() {
    switch (servicer.id) {
      case ServicerID.GREAT_LAKES:
        return _applyGreatLakesRules();
      case ServicerID.EARNEST:
        return _applyEarnestRules();
    }
  }

  _applyGreatLakesRules() {
    _isUserIDInsteadOfUserName.value = true;
  }

  _applyEarnestRules() => controllerAuth.authStep.value =
      controllerPolledAuth?.authStep.value ?? AuthStep.STEP_UNAUTHORIZED;

  Widget _initialState(bool hasError) {
    controllerAuth.authStep.value = AuthStep.STEP_UNAUTHORIZED;
    // canLaunchURL(servicer.forgotPasswordUrl).then((v) => _canLaunchForgotPassword.value = v);
    return Scaffold(
      backgroundColor: colorScheme.background,
      bottomNavigationBar: _continueButtonInitialState(),
      body: Flex(
        direction: Axis.vertical,
        mainAxisSize: MainAxisSize.max,
        children: [
          SlimAppBar(''),
          Expanded(
              child: SingleChildScrollView(
                  controller: scrollController, child: _loginFields(hasError))),
        ],
      ),
    );
  }

  Widget _continueButtonInitialState() => Observer(() => IgnorePointer(
      ignoring: _sendButtonClicked.value,
      child: BetterButton(
          text: "Connect loan",
          onPressed: () async {
            _sendButtonClicked.value = true;
            await _sendRequestAuth();
          },
          isLoading: controllerAuth.isPolledTask.value && controllerPolledAuth!.isPolling.value ||
              _sendButtonClicked.value)));

  Future _sendRequestAuth() async {
    bool ssn = false;
    bool accountNumber = false;
    bool dob = false;
    bool pin = false;
    servicer.authSteps[0].authParams.forEach((element) {
      if (element.id == "ssn") {
        ssn = true;
      }
      if (element.id == "accountNumber") {
        accountNumber = true;
      }
      if (element.id == "dob") {
        dob = true;
      }
      if (element.id == "pin") {
        pin = true;
      }
    });

    await controllerAuth.requestAuth(
      RequestAuth(
        loginDetails: UserPass(
          username: _userBox.value.field.controller!.text,
          userId: _userBox.value.field.controller!.text,
          password: _passwordBox.field.controller!.text,
          pin: pin ? _authParams[1].text : null,
          dob: dob ? _datePicker.selectedDate : null,
          ssn: ssn ? _authParams[3].text : null,
          accountNumber: accountNumber
              ? _authParams[4].text.isNotEmpty
                  ? _authParams[4].text
                  : _authParams[3].text
              : null,
        ),
        authStep: 'password',
        loanServicerId: servicer.id,
        extUserID: getSafe<IServiceToken>()?.requestToken?.extUserId,
      ),
    );
    _sendButtonClicked.value = false;
  }

  Widget _authStepSecurity(BuildContext context, bool hasError) {
    controllerAuth.authStep.value = AuthStep.STEP_SECURITY;
    final SecurityStep security = controllerAuth.security.value;

    List<BlueItemFormField> formItems = security.securityDetails
        .asMap()
        .entries
        .map(
          (entry) => BlueItemFormField(
            label: entry.value.question,
            initialValue: entry.value.answer,
          ),
        )
        .toList();

    return Center(
      child: Scaffold(
        backgroundColor: colorScheme.background,
        appBar: SlimAppBar('', onBackPressed: () => _onBackPressed(context)),
        bottomNavigationBar: _continueButtonSecurityQuestions(security, formItems),
        body: Center(
          child: SingleChildScrollView(
            controller: scrollController,
            child: Column(
              mainAxisAlignment: MainAxisAlignment.spaceEvenly,
              children: [
                Container(
                  margin: EdgeInsets.only(left: 16),
                  child: _servicerLogo(),
                ),
                hasError
                    ? _createErrorBox(
                        margin: EdgeInsets.symmetric(horizontal: 32),
                        message: 'One or more answers are incorrect. Kindly check & try again.')
                    : SizedBox(),
                ...formItems,
                const SizedBox(height: 16),
              ],
            ),
          ),
        ),
      ),
    );
  }

  Widget _continueButtonSecurityQuestions(
          SecurityStep security, List<BlueItemFormField> formItems) =>
      Observer(
        () => IgnorePointer(
          ignoring: _sendButtonClicked.value,
          child: BetterButton(
              text: "Continue",
              isLoading: _sendButtonClicked.value,
              onPressed: () {
                _requestAuthAddSteps(
                    security,
                    AuthStepParams(accountUserName: security.nextAuthStepParams.accountUserName),
                    LoginSecurityDetails(
                      securityDetails: List.generate(
                        security.securityDetails.length,
                        (index) => SecurityDetails(
                          questionId: security.securityDetails[index].questionId,
                          question: security.securityDetails[index].question,
                          answer: formItems[index].field.controller?.text,
                        ),
                      ),
                    ));
                _sendButtonClicked.value = true;
              }),
        ),
      );

  Future _requestAuthAddSteps(
    SecurityStep security,
    AuthStepParams authStepParams,
    LoginSecurityDetails loginSecurityDetails,
  ) async {
    await controllerAuth.requestAuthAddSteps(RequestAuthAddSteps(
      userId: security.userId,
      authStep: security.nextAuthStep,
      authStepParams: authStepParams,
      loanServicerId: servicer.id,
      loginDetails: loginSecurityDetails,
    ));
    _sendButtonClicked.value = false;
  }

  Widget _authCompleted(BuildContext context) => ResultScreen(
        hasExitHeader: true,
        onExit: getSafe<EventHandlerLoanConnect>()?.onExit ?? () => _onBackPressed(context),
        icon: ImageCompat(
          'success.svg',
          package: 'sw_loan_connect',
          width: 60,
          height: 60,
        ),
        title: 'Student loan connected',
        firstButtonText: 'Connect another loan',
        firstButtonPressed: getSafe<EventHandlerLoanConnect>()?.onConnectAnotherLoan ??
            () => _onBackPressed(context),
        secondButtonText: 'View my account',
        secondButtonPressed: getSafe<EventHandlerLoanConnect>()?.onContinue ??
            () => ScaffoldMessenger.of(context).showSnackBar(
                  SnackBar(
                    behavior: SnackBarBehavior.floating,
                    content: Text("Replace me with a navigation callback!"),
                  ),
                ),
      );

  Widget _loginFields(bool hasError) => Flex(
        direction: Axis.vertical,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          _servicerLogo(),
          Padding(
            padding: EdgeInsets.only(left: 16),
            child: Text(
              'Enter your loan information',
              style: theme.textTheme.headline4,
              textAlign: TextAlign.left,
            ),
          ),
          Observer(() => hasError || controllerPolledAuth!.error.value.code != null
              ? _createErrorBox()
              : SizedBox()),
          SizedBox(height: 32),
          _changeUserBoxLabel(),
          _passwordBox,
          ..._authParamsFields(),
          _forgotPasswordLink(),
          const SizedBox(height: 64),
        ],
      );

  _changeUserBoxLabel() => _isUserIDInsteadOfUserName.value
      ? _userBox.value = BlueItemFormField(label: 'User ID')
      : _userBox.value;

  Widget _createErrorBox({String? message, EdgeInsets? margin}) => Center(
        child: Container(
          margin: margin ?? EdgeInsets.only(left: 16, right: 16, top: 20),
          child: Visibility(
            visible: _errorBoxVisibility.value,
            child: ErrorBox(
              () => setVisibility(false),
              message: servicer.authSteps[0].authParams.length > 2
                  ? "Invalid credentials. Kindly recheck and try again."
                  : message ?? null,
              width: double.infinity,
            ),
          ),
        ),
      );

  void setVisibility(bool value) => _errorBoxVisibility.value = value;

  Widget _servicerLogo() => Container(
        alignment: Alignment.centerLeft,
        padding: const EdgeInsets.only(top: 21, bottom: 27, left: 16),
        child: ImageCompat(
          _assets.containsKey(servicer.id) ? _assets[servicer.id]! : servicer.logoUrl.toString(),
          height: 60,
          width: 80,
          fit: BoxFit.fitWidth,
          package: _assets.containsKey(servicer.id) ? "sw_loan_connect" : '',
        ),
      );

  _authParamsFields() {
    _authParams.clear();

    _createTxtControllers(servicer);

    final fields = _mapAuthStepsFields((
      param,
    ) =>
        servicer.authSteps[0].authParams.length > 4 && param.key == 3
            ? _checkAddORLabel(param)
            : _checkAddDatePicker(param)).toList();

    return fields;
  }

  _mapAuthStepsFields(Function(MapEntry<int, AuthParams> param) callback) {
    if (servicer.authSteps.isNotEmpty) {
      return servicer.authSteps[0].authParams
          .asMap()
          .entries
          .where((element) => element.value.group != "1")
          .where((element) => element.value.group != "2")
          .map(callback);
    }
    return <Widget>[];
  }

  _checkAddORLabel(MapEntry<int, AuthParams> param) => Column(
        children: [
          RawBlueItemFormField(
              label: param.value.label,
              showHint: true,
              controller: param.key <= servicer.authSteps[0].authParams.length
                  ? _authParams[param.key]
                  : null),
          Container(
              margin: EdgeInsets.only(top: 16),
              child: Text(
                "OR",
                style: theme.textTheme.headline4,
                textAlign: TextAlign.center,
              )),
        ],
      );

  _checkAddDatePicker(MapEntry<int, AuthParams> param) =>
      servicer.authSteps[0].authParams.length > 4 && param.key == 2
          ? _datePicker
          : servicer.authSteps[0].authParams.length >= 4 && param.key == 2
              ? _datePicker
              : RawBlueItemFormField(
                  label: param.value.label,
                  showHint: false,
                  controller: param.key <= servicer.authSteps[0].authParams.length
                      ? _authParams[param.key]
                      : null);

  void _createTxtControllers(Servicer servicer) {
    if (servicer.authSteps.isNotEmpty) {
      int paramsQtd = servicer.authSteps[0].authParams.length;

      for (int i = 0; i <= paramsQtd; i++) {
        _authParams.add(TextEditingController());
      }
    }
  }

  Widget _forgotPasswordLink() => Observer(
        () => _canLaunchForgotPassword.value
            ? Padding(
                padding: const EdgeInsets.only(
                  left: 16,
                  top: 4,
                  right: 16,
                  bottom: 16,
                ),
                child: GestureDetector(
                  onTap: () {
                    launchURL(servicer.forgotPasswordUrl);
                  },
                  child: Text(
                    'Forgot password?',
                    style: TextStyle(
                      color: colorScheme.primary,
                      fontStyle: theme.textTheme.bodyText1!.fontStyle,
                      fontSize: theme.textTheme.bodyText1!.fontSize,
                      fontWeight: theme.textTheme.bodyText1!.fontWeight,
                      fontFamily: theme.textTheme.bodyText1!.fontFamily,
                      height: theme.textTheme.bodyText1!.height,
                      locale: theme.textTheme.bodyText1!.locale,
                      textBaseline: theme.textTheme.bodyText1!.textBaseline,
                    ),
                  ),
                ),
              )
            : Center(),
      );

  _scrollToTop() => scrollController.animateTo(0,
      duration: Duration(milliseconds: 500), curve: Curves.fastOutSlowIn);

  _authError(BuildContext context) {
    setVisibility(true);
    _scrollToTop();
    return controllerAuth.security.value.securityDetailsRequired != null &&
            controllerAuth.security.value.securityDetailsRequired!
        ? _authStepSecurity(context, true)
        : _initialState(true);
  }
}
