import 'package:flutter/material.dart';
import 'package:sw_component/components/widget/date_picker.dart';
import 'package:sw_component/components/widget/error_box.dart';
import 'package:sw_component/components/widget/image_compat.dart';
import 'package:sw_component/components/widget/item_security_question.dart';
import 'package:sw_component/components/widget/material_theme_wrapper.dart';
import 'package:sw_component/components/widget/slim_appbar.dart';
import 'package:sw_core/entity_auth/auth_step_params.dart';
import 'package:sw_core/entity_auth/login_security_details.dart';
import 'package:sw_core/entity_auth/password_step.dart';
import 'package:sw_core/entity_auth/security_details.dart';
import 'package:sw_core/entity_auth/user_pass.dart';
import 'package:sw_core/entity_servicer/auth_params.dart';
import 'package:sw_core/entity_servicer/servicer.dart';
import 'package:sw_core/entity_theme/params_loan_connect.dart';
import 'package:sw_core/interface/controller/i_controller_all_servicers.dart';
import 'package:sw_core/interface/controller/i_controller_auth.dart';
import 'package:sw_core/interface/controller/i_controller_polled_auth.dart';
import 'package:sw_core/interface/service/i_service_token.dart';
import 'package:sw_core/request/request_auth.dart';
import 'package:sw_core/request/request_auth_add_steps.dart';
import 'package:sw_core/tool/inject.dart';
import 'package:sw_core/navigation/nav.dart';
import 'package:sw_core/tool/observer.dart';
import 'package:sw_core/tool/state.dart';
import 'package:sw_loan_connect/tool/servicer_id.dart';

class LayoutLoginDefault extends StatelessWidget {
  LayoutLoginDefault({Key? key}) : super(key: key);

  final ParamsLoanConnect params = get<ParamsLoanConnect>();
  final Servicer servicer = get<Servicer>();

  final controllerAuth = get<IControllerAuth>();
  final controllerPolledAuth = get<IControllerPolledAuth>();
  final controllerAllServicers = get<IControllerAllServicers>();
  final DatePicker _datePicker = DatePicker();
  final serviceToken = get<IServiceToken>();
  final _userBox = ItemFormField(
    showHint: true,
    hint: "Username",
  ).obs;
  final ItemFormField _passwordBox = ItemFormField(
    hint: "Password",
    showHint: true,
    obscureText: true,
  );
  final scrollController = ScrollController();
  final List<TextEditingController> _authParams = [];
  final List<Widget> _inGreyContainer = [];
  final Map<String, String> _assets = {
    "9422554f-0170-4afe-b8df-12aaecc3fdef": "logo_aes.png",
    "e32be248-d43f-4dd4-9e34-05c7b8ab1888": "logo_firstmark.png",
    "bb3e39e7-9dd6-461e-a4f4-c95a1a1e80ef": "logo_nelnet.png",
    "e35a6454-ca66-4b35-a7e7-afb25d03c424": "logo_suntrust.png",
  };
  final String sallieMaeId = "09945206-21fc-4e8b-9779-be328c0926d5";

  //rules
  final _isUserIDInsteadOfUserName = false.obs;
  final _canLaunchForgotPassword = true.obs;
  final _errorBoxVisibility = false.obs;

  BuildContext? context;

  @override
  Widget build(BuildContext context) {
    this.context = context;
    return MaterialThemeWrapper(
      child: Scaffold(
        backgroundColor: colorScheme.background,
        appBar: _buildAppBar(),
        body: _buildBody(),
      ),
    );
  }

  AppBar _buildAppBar() => SlimAppBar('We are ready to GO!', onBackPressed: _onBackPressed);

  Widget _buildBody() => Observer(_checkAuthStep);

  _onBackPressed() {
    controllerAuth.authStep.value = AuthStep.STEP_UNAUTHORIZED;
    back(context);
  }

  Widget _checkAuthStep() {
    _applyRules();
    switch (controllerAuth.authStep.value) {
      case AuthStep.STEP_UNAUTHORIZED:
        return _initialState(false);
      case AuthStep.STEP_SECURITY:
        return _authStepSecurity(false);
      case AuthStep.STEP_AUTHORIZED:
        return _authCompleted();
      case AuthStep.AUTH_ERROR:
        return _authError();
    }
  }

  _applyRules() {
    switch (servicer.id) {
      case ServicerID.GREAT_LAKES:
        return _applyGreatLakesRules();
      case ServicerID.EARNEST:
        return _applyEarnestRules();
    }
  }

  _applyGreatLakesRules() {
    _isUserIDInsteadOfUserName.value = true;
  }

  _applyEarnestRules() => controllerAuth.authStep.value = controllerPolledAuth.authStep.value;

  Widget _initialState(bool hasError) {
    // canLaunchURL(servicer.forgotPasswordUrl).then((v) => _canLaunchForgotPassword.value = v);
    return Column(
      children: [
        Expanded(
            child: SingleChildScrollView(
                controller: scrollController, child: Center(child: _loginFields(hasError)))),
        SizedBox(
          height: 64,
          width: double.infinity,
          child: _continueButtonInitialState(),
        ),
      ],
    );
  }

  Widget _continueButtonInitialState() => ElevatedButton(
        style: ElevatedButton.styleFrom(
          onPrimary: colorScheme.secondary,
          onSurface: colorScheme.secondary,
        ),
        onPressed: _sendRequestAuth,
        child: Text(
          'Continue',
          style: TextStyle(
            color: colorScheme.onPrimary,
          ),
        ),
      );

  void _sendRequestAuth() {
    bool ssn = false;
    bool accountNumber = false;
    bool dob = false;
    bool pin = false;
    servicer.authSteps[0].authParams.forEach((element) {
      if (element.id == "ssn") {
        ssn = true;
      }
      if (element.id == "accountNumber") {
        accountNumber = true;
      }
      if (element.id == "dob") {
        dob = true;
      }
      if (element.id == "pin") {
        pin = true;
      }
    });

    controllerAuth.requestAuth(
      RequestAuth(
        loginDetails: UserPass(
          username: _userBox.value.field.controller!.text,
          userId: _userBox.value.field.controller!.text,
          password: _passwordBox.field.controller!.text,
          pin: pin ? _authParams[1].text : null,
          dob: dob ? _datePicker.selectedDate : null,
          ssn: ssn ? _authParams[3].text : null,
          accountNumber: accountNumber
              ? _authParams[4].text.isNotEmpty
                  ? _authParams[4].text
                  : _authParams[3].text
              : null,
        ),
        authStep: 'password',
        loanServicerId: servicer.id,
        extUserID: getSafe<IServiceToken>()?.requestToken?.extUserId,
      ),
    );
  }

  Widget _authStepSecurity(bool hasError) {
    final SecurityStep security = controllerAuth.security.value;

    List<ItemFormField> formItems = security.securityDetails
        .asMap()
        .entries
        .map(
          (entry) => ItemFormField(
            label: entry.value.question,
            icon: Icons.lock_outline,
            initialValue: entry.value.answer,
          ),
        )
        .toList();

    return Center(
      child: Scaffold(
        backgroundColor: colorScheme.background,
        bottomNavigationBar: _continueButtonSecurityQuestions(security, formItems),
        body: Center(
          child: SingleChildScrollView(
            controller: scrollController,
            child: Column(
              mainAxisAlignment: MainAxisAlignment.spaceEvenly,
              children: [
                hasError
                    ? _createErrorBox(
                        margin: EdgeInsets.symmetric(horizontal: 32),
                        message: 'One or more answers are incorrect. Kindly check & try again.')
                    : SizedBox(),
                ...formItems,
                const SizedBox(height: 16),
              ],
            ),
          ),
        ),
      ),
    );
  }

  Widget _continueButtonSecurityQuestions(SecurityStep security, List<ItemFormField> formItems) =>
      Container(
        height: 64,
        child: ElevatedButton(
          style: ElevatedButton.styleFrom(
            onPrimary: colorScheme.secondary,
            onSurface: colorScheme.secondary,
          ),
          onPressed: () => _requestAuthAddSteps(
              security,
              AuthStepParams(accountUserName: security.nextAuthStepParams.accountUserName),
              LoginSecurityDetails(
                securityDetails: List.generate(
                  security.securityDetails.length,
                  (index) => SecurityDetails(
                    questionId: security.securityDetails[index].questionId,
                    question: security.securityDetails[index].question,
                    answer: formItems[index].field.controller?.text,
                  ),
                ),
              )),
          child: Text(
            'Continue',
            style: TextStyle(
              color: colorScheme.onPrimary,
            ),
          ),
        ),
      );

  _requestAuthAddSteps(
    SecurityStep security,
    AuthStepParams authStepParams,
    LoginSecurityDetails loginSecurityDetails,
  ) =>
      controllerAuth.requestAuthAddSteps(RequestAuthAddSteps(
        userId: security.userId,
        authStep: security.nextAuthStep,
        authStepParams: authStepParams,
        loanServicerId: servicer.id,
        loginDetails: loginSecurityDetails,
      ));

  Widget _authCompleted() => Center(
        child: Column(
          mainAxisAlignment: MainAxisAlignment.spaceEvenly,
          children: [
            const SizedBox(
              height: 32,
            ),
            const Center(
              child: Text(
                'Authentication\nComplete',
                textAlign: TextAlign.center,
                style: TextStyle(fontSize: 38),
              ),
            ),
            const SizedBox(
              height: 32,
            ),
            Text(
              '\nextUserID: ${controllerAuth.user.value.extUserId}\nservicerID: ${servicer.id}',
              textAlign: TextAlign.start,
              style: const TextStyle(fontSize: 14),
            ),
            const SizedBox(
              height: 32,
            ),
          ],
        ),
      );

  Widget _loginFields(bool hasError) => Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          _servicerLogo(),
          const Padding(
            padding: EdgeInsets.only(left: 50),
            child: Text(
              'Enter your loan information',
              style: TextStyle(fontSize: 20),
              textAlign: TextAlign.center,
            ),
          ),
          hasError ? _createErrorBox() : SizedBox(),
          SizedBox(height: 32),
          _changeUserBoxLabel(),
          _passwordBox,
          ..._authParamsFields(),
          SizedBox(height: 6),
          _forgotPasswordLink(),
          const SizedBox(height: 64)
        ],
      );

  _changeUserBoxLabel() => _isUserIDInsteadOfUserName.value
      ? _userBox.value = ItemFormField(showHint: true, hint: 'User ID')
      : _userBox.value;

  Widget _createErrorBox({String? message, EdgeInsets? margin}) => Center(
        child: Observer(() => Container(
              margin: margin ?? EdgeInsets.only(left: 16, right: 16, top: 20),
              child: Visibility(
                visible: _errorBoxVisibility.value,
                child: ErrorBox(
                  () => setVisibility(false),
                  message: message ?? null,
                  width: double.infinity,
                ),
              ),
            )),
      );

  void setVisibility(bool value) => _errorBoxVisibility.value = value;

  Widget _servicerLogo() => Container(
        alignment: Alignment.center,
        padding: const EdgeInsets.all(50),
        child: ImageCompat(
          _assets.containsKey(servicer.id) ? _assets[servicer.id]! : servicer.logoUrl.toString(),
          height: 240,
          fit: BoxFit.fitWidth,
          package: _assets.containsKey(servicer.id) ? "sw_loan_connect" : '',
        ),
      );

  _authParamsFields() {
    _authParams.clear();
    _inGreyContainer.clear();

    _createTxtControllers(servicer);

    List fields = _mapAuthStepsFields((param) {
      _addOrderedElements(param);
      return _checkAddGrayContainer(param);
    }).toList();

    if (servicer.authSteps[0].authParams.length >= 4 && servicer.id! != sallieMaeId)
      fields.removeLast();

    return fields;
  }

  _mapAuthStepsFields(Function(MapEntry<int, AuthParams>) callback) {
    if (servicer.authSteps.isNotEmpty) {
      return servicer.authSteps[0].authParams
          .asMap()
          .entries
          .where((element) => element.value.group != "1")
          .where((element) => element.value.group != "2")
          .map(callback);
    }
    return <Widget>[];
  }

  _checkAddDatePicker(MapEntry<int, AuthParams> param) =>
      servicer.authSteps[0].authParams.length > 4 && param.key == 2
          ? _datePicker
          : servicer.authSteps[0].authParams.length >= 4 && param.key == 2
              ? _datePicker
              : RawItemFormField(
                  hint: param.value.label,
                  showHint: true,
                  controller: param.key <= servicer.authSteps[0].authParams.length
                      ? _authParams[param.key]
                      : null);

  _checkAddGrayContainer(MapEntry<int, AuthParams> param) =>
      servicer.authSteps[0].authParams.length > 4 && param.key == 3
          ? Container(
              margin: EdgeInsets.only(top: 16, left: 32, right: 32),
              padding: EdgeInsets.all(32),
              decoration: BoxDecoration(
                  color: Colors.grey, borderRadius: BorderRadius.all(Radius.circular(32))),
              child: Column(
                mainAxisAlignment: MainAxisAlignment.start,
                children: _inGreyContainer,
              ),
            )
          : _checkAddDatePicker(param);

  _addOrderedElements(MapEntry<int, AuthParams> param) {
    if (servicer.authSteps[0].authParams.length > 4) {
      switch (param.key) {
        case 3:
          _inGreyContainer.add(RawItemFormField(
              enableMargins: false,
              hint: param.value.label,
              showHint: true,
              controller: _checkParamKey(param) ? _authParams[param.key] : null));
          _inGreyContainer.add(Container(
              margin: EdgeInsets.only(top: 16),
              child: Text(
                "OR",
                style: theme.textTheme.headline4,
                textAlign: TextAlign.center,
              )));
          break;
        case 4:
          _inGreyContainer.add(RawItemFormField(
              enableMargins: false,
              hint: param.value.label,
              showHint: true,
              controller: _checkParamKey(param) ? _authParams[param.key] : null));
          break;
      }
    }
  }

  _checkParamKey(MapEntry<int, AuthParams> param) =>
      param.key <= servicer.authSteps[0].authParams.length;

  void _createTxtControllers(Servicer servicer) {
    if (servicer.authSteps.isNotEmpty) {
      int paramsQtd = servicer.authSteps[0].authParams.length;

      for (int i = 0; i <= paramsQtd; i++) {
        _authParams.add(TextEditingController());
      }
    }
  }

  Widget _forgotPasswordLink() => Observer(
        () => _canLaunchForgotPassword.value
            ? Padding(
                padding: const EdgeInsets.only(left: 32, top: 10, right: 50),
                child: GestureDetector(
                    onTap: () {
                      launchURL(servicer.forgotPasswordUrl);
                    },
                    child: Text('Forgot password?', style: TextStyle(color: colorScheme.primary))),
              )
            : Center(),
      );

  _scrollToTop() => scrollController.animateTo(0,
      duration: Duration(milliseconds: 500), curve: Curves.fastOutSlowIn);

  Widget _authError() {
    setVisibility(true);
    _scrollToTop();
    return controllerAuth.security.value.securityDetailsRequired != null &&
            controllerAuth.security.value.securityDetailsRequired!
        ? _authStepSecurity(true)
        : _initialState(true);
  }
}
