import 'package:flutter/material.dart';
import 'package:sw_component/components/widget/item_form_field.dart';
import 'package:sw_component/components/widget/item_servicer.dart';
import 'package:sw_component/components/widget/material_theme_wrapper.dart';
import 'package:sw_component/components/widget/slim_appbar.dart';
import 'package:sw_core/entity_servicer/servicer.dart';
import 'package:sw_core/entity_theme/params_loan_connect.dart';
import 'package:sw_core/interface/controller/i_controller_all_servicers.dart';
import 'package:sw_core/tool/inject.dart';
import 'package:sw_core/navigation/nav.dart';
import 'package:sw_core/tool/observer.dart';
import 'package:sw_core/tool/state.dart';
import 'package:sw_loan_connect/layouts/layout_unintegrated_login_screen_b.dart';
import 'package:sw_loan_connect/routes/route_login.dart';

class LayoutServicersB extends StatelessWidget {
  LayoutServicersB({Key? key}) : super(key: key);

  final ParamsLoanConnect params = get<ParamsLoanConnect>();
  final controller = get<IControllerAllServicers>();
  final search = ''.obs;
  BuildContext? context;

  final Map<String, String> _assets = {
    "9422554f-0170-4afe-b8df-12aaecc3fdef": "logo_aes.png",
    "e32be248-d43f-4dd4-9e34-05c7b8ab1888": "logo_firstmark.png",
    "bb3e39e7-9dd6-461e-a4f4-c95a1a1e80ef": "logo_nelnet.png",
    "e35a6454-ca66-4b35-a7e7-afb25d03c424": "logo_suntrust.png",
  };

  @override
  Widget build(BuildContext context) {
    this.context = context;
    controller.getAllServicers();
    return MaterialThemeWrapper(
        child: Scaffold(
      backgroundColor: colorScheme.background,
      appBar: SlimAppBar(
        'Select your loan servicer',
        style: theme.textTheme.bodyText1,
        centerTitle: true,
        onBackPressed: () => back(context),
      ),
      body: _body(),
    ));
  }

  Widget _body() => Column(
        children: [
          Row(
            children: [
              Expanded(
                child: _searchbar(),
              ),
            ],
          ),
          Flexible(
            child: Observer(() => _loadServicerList().length <= 0
                ? _loanNotFound()
                : Padding(
                    padding: const EdgeInsets.fromLTRB(16, 0, 16, 16),
                    child: GridView.count(
                      crossAxisCount: 2,
                      shrinkWrap: true,
                      mainAxisSpacing: 16,
                      crossAxisSpacing: 16,
                      childAspectRatio: 2.2 / 1,
                      children: _loadServicerList(),
                    ),
                  )),
          )
        ],
      );

  Widget _searchbar() => Container(
        margin: EdgeInsets.only(bottom: 32),
        child: BlueItemFormField(
            label: 'Search', onChanged: (string) => search.value = string.toLowerCase()),
      );

  _loanNotFound() => Center(
        child: InkWell(
          onTap: () => routeUnintegrated(context),
          splashFactory: NoSplash.splashFactory,
          highlightColor: Colors.transparent,
          child: Text(
            "Don't see your loan\nservicer?",
            style: TextStyle(
              color: colorScheme.primary,
              fontStyle: theme.textTheme.bodyText1!.fontStyle,
              fontSize: theme.textTheme.bodyText1!.fontSize,
              fontWeight: theme.textTheme.bodyText1!.fontWeight,
              fontFamily: theme.textTheme.bodyText1!.fontFamily,
              height: theme.textTheme.bodyText1!.height,
              locale: theme.textTheme.bodyText1!.locale,
              textBaseline: theme.textTheme.bodyText1!.textBaseline,
            ),
            textAlign: TextAlign.center,
          ),
        ),
      );

  List<ItemServicer> _loadServicerList() {
    List<ItemServicer> servicers = controller.allServicers
        .map(
          (servicer) => ItemServicer(
            servicer: servicer,
            onTap: (servicer) {
              replace<Servicer>(() => servicer);
              routeLogin(context);
            },
          ),
        )
        .where(_integrated)
        .where(_search)
        .toList();

    servicers = _changeAsset(servicers);

    return servicers;
  }

  bool _search(ItemServicer element) {
    if (search.value.trim().isEmpty) {
      return true;
    }
    return element.servicer.name.toLowerCase().contains(search.value);
  }

  bool _integrated(ItemServicer element) => element.servicer.isIntegrated;

  _changeAsset(List<ItemServicer> servicers) => servicers
      .asMap()
      .map((index, itemServicer) => MapEntry(
          index,
          _assets.containsKey(itemServicer.servicer.id)
              ? itemServicer = ItemServicer(
                  onTap: itemServicer.onTap,
                  localImage: _assets[itemServicer.servicer.id],
                  servicer: itemServicer.servicer,
                )
              : itemServicer))
      .values
      .toList();
}
